/*
 *  dvtext.c
 *
 *  Copyright (C) 2004 David Simmons
 *
 *  This file is part of dvtext.
 *
 *  dvtext is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  dvtext is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with dvtext; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <stdio.h>
#include <getopt.h>
#include <errno.h>
#include "dvtext.h"
#include "fonts.h"

/*
 * parse()
 *
 * Input:
 *	dvtext = The dvtext handle of this instance
 *	input_filename = the dvtext description file to read
 */

int parse(dvtext_t *dvtext, char *input_filename) {
	FILE *infile;
	char linebuffer[256];
	int lineno=0;
	textitem_t *current_settings;
	char *s;

	/* create the "current textitem defaults" node */
	current_settings = (textitem_t*)malloc(sizeof(textitem_t));
	memset(current_settings, 0, sizeof(textitem_t));
	current_settings->fontname = strdup(DEFAULT_FONT);
	current_settings->pointsize = DEFAULT_POINTSIZE;
	current_settings->ypos = DEFAULT_YPOS;

	/* open the input file */
	infile = fopen(input_filename, "r");
	if (infile == NULL) {
		fprintf(stderr, "Cannot open input file \"%s\": %s\n",
			input_filename, strerror(errno));
		return -1;
	}

	/* iterate over each line */
	while (fgets(linebuffer, 256, infile)) {
		lineno++;

		/* trim any trailing newline */
		if (linebuffer[strlen(linebuffer)-1] == '\n') {
			linebuffer[strlen(linebuffer)-1] = '\0';
		}

		/* take action based on the first character of the line */
		if (linebuffer[0] == '#') {
			/* comment -- do nothing */
		} else if (linebuffer[0] == '!') {
			char *cmd;
			char *arg;

			cmd = strtok(linebuffer+1, " \t");
			if (cmd == NULL) {
				fprintf(stderr, "line %d: invalid line: \"%s\"\n",lineno,linebuffer);
				continue;
			}
			arg = strtok(NULL, " \t");

			if (strcmp(cmd, "type")==0) {
				if (!arg) {
					fprintf(stderr, "line %d: 'type' requires argument.\n", lineno);
					continue;
				}
				if (strcmp(arg,"title")==0) {
					dvtext->texttype = TEXTTYPE_TITLE;
				} else if (strcmp(arg, "scroll")==0) {
					dvtext->texttype = TEXTTYPE_SCROLL;
				} else {
					fprintf(stderr, "line %d: invalid argument to 'type'.\n", lineno);
					continue;
				}
			} else if (strcmp(cmd, "font")==0) {
				if (!arg) {
					fprintf(stderr, "line %d: 'font' requires argument.\n", lineno);
					continue;
				}
				current_settings->fontname = strdup(arg);
			} else if (strcmp(cmd, "pointsize")==0) {
				if (!arg) {
					fprintf(stderr, "line %d: 'pointsize' requires argument.\n", lineno);
					continue;
				}
				current_settings->pointsize = atoi(arg);
			} else if (strcmp(cmd, "ypos")==0) {
				if (!arg) {
					fprintf(stderr, "line %d: 'ypos' requires argument.\n", lineno);
					continue;
				}
				current_settings->ypos = atoi(arg);
			} else {
				fprintf(stderr,"line %d: unknown command\n", lineno);
			}
			/*  add leadin fadein solid fadeout leadout  */
			/*  add scroll, pointsize */
		} else {

			/* allocate a new textitem_t node */
			textitem_t *node = (textitem_t*)malloc(sizeof(textitem_t));
			memset(node, 0, sizeof(textitem_t));

			/* fill it with current settings */
			node->fontname = strdup(current_settings->fontname);
			node->pointsize = current_settings->pointsize;
			node->ypos = current_settings->ypos;

			/* copy the current text line into it */
			node->string = strdup(linebuffer);

			/* link it to the list */

			if (dvtext->textitem_list == NULL) {
				/* empty list -- this node becomes the head */
				dvtext->textitem_list = node;
			} else {
				/* find the tail of the list */
				textitem_t *tail;
				for (tail=dvtext->textitem_list; tail->next!=NULL; tail=tail->next) {}
				/* link the new node to the tail */
				tail->next = node;
			}
		}

	}
	fclose(infile);

}


/*
 * int standard_init(standard_t **st)
 *
 * Initializing standard vector
 */

int standard_init(standard_t **st) {

	standard_t *standard = NULL;

	if((standard = (standard_t *)malloc(sizeof(standard_t)*3)) == NULL) 
		return (-1);
		
	memset(standard, 0, sizeof(standard_t)*3);
	
	(standard+NTSC)->framerate = 29.97;
	(standard+NTSC)->width = NTSC_WIDTH;
	(standard+NTSC)->height = NTSC_HEIGHT;
	(standard+NTSC)->framesize = DV_FRAME_SIZE_NTSC;
	
	(standard+PAL)->framerate = 25.;
	(standard+PAL)->width = PAL_WIDTH;
	(standard+PAL)->height = PAL_HEIGHT;
	(standard+PAL)->framesize = DV_FRAME_SIZE_PAL;
	
	(standard+SECAM)->framerate = 25.;
	(standard+SECAM)->width = PAL_WIDTH;
	(standard+SECAM)->height = PAL_HEIGHT;
	(standard+SECAM)->framesize = DV_FRAME_SIZE_PAL;
	
	
		
	*st = standard;
	
	return (0);
}

/*
 * usage()
 *
 * Print a usage summary
 */

void usage() {
	printf("usage: dvtext NTSC|PAL|SECAM filename.dvt\n");
}


/*
 * main()
 *
 * Parse command-line arguments, initialize libraries,
 * populate a dvtext_t structure with information about
 * this job, and invoke the required render routine.
 */

main(int argc, char** argv) {
	dvtext_t *dvtext;
	standard_t *standard;
	standardtype_t type;

/* 	while (1) {
		int c = getopt(argc, argv, "");
		if (c == -1) {
			break;
		}

		switch (c) {
		default:
			usage();
			exit(1);
		}
	} 

	if ((argc - optind) != 2) {
		usage();
		exit(1);
	}*/
	
	if(argc != 3)
	{
		usage();
		exit(1);		
	}
	
	if( strcmp("NTSC", argv[1]) == 0) 
	{
		type = NTSC;
	}
	else if( strcmp("PAL", argv[1]) == 0) 
	{
		type = PAL;
	}
	else if( strcmp("SECAM", argv[1]) == 0)
	{
		type = SECAM;
	}
	else
	{
		usage();
		exit(1);
	}
	printf("\nTipo: %d %s\n\n", type, argv[2]);	

	dvtext = (dvtext_t *)malloc(sizeof(dvtext_t));
	memset(dvtext, 0, sizeof(dvtext_t));
	char *input_filename = argv[2];

	/* set global defaults */
	dvtext->texttype	= DEFAULT_TEXTTYPE;
	dvtext->title_leadin	= DEFAULT_TITLE_LEADIN;
	dvtext->title_fadein	= DEFAULT_TITLE_FADEIN;
	dvtext->title_solid	= DEFAULT_TITLE_SOLID;
	dvtext->title_fadeout	= DEFAULT_TITLE_FADEOUT;
	dvtext->title_leadout	= DEFAULT_TITLE_LEADOUT;

	/* initialize freetype */
	if (fonts_init(dvtext) == -1) { 
		exit(1);
	}

	/* initialize standard data */
	if (standard_init(&standard) == -1) { 
		exit(1);
	}
	
	/* parse the input file */
	if (parse(dvtext, input_filename) == -1) {
		exit(1);
	}

	/* load fonts referenced in the input file */
	if (fonts_load(dvtext) == -1) {
		exit(1);
	}

	/* measure the lengths of the strings */
        textitem_t *item;
        for (item=dvtext->textitem_list; item!=NULL; item=item->next) {
		item->pixel_width = fonts_string_measure(item->textface->face, item->string);
	}
 
	/* select a renderer */
	if (dvtext->texttype == TEXTTYPE_TITLE) {
		if (render_title(dvtext, standard, type) == -1) {
			exit(1);
		}
	} else if (dvtext->texttype == TEXTTYPE_SCROLL) {
		if (render_scroll(dvtext, standard, type) == -1) {
			exit(1);
		}
	}

	

	/* TODO: free freetype */
	
	if(standard) free(standard);
	
	free(dvtext);
}

