/*
 *  render.c
 *
 *  Copyright (C) 2004 David Simmons
 *
 *  This file is part of dvtext.
 *
 *  dvtext is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  dvtext is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with dvtext; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <libdv/dv.h>
#include <avilib.h>
#include "dvtext.h"
#include "render.h"

/*
 * render_title()
 *
 * Input:
 *	dvtext = the dvtext_t handle for this job
 *
 * Render a fade-in/fade-out title effect.
 */

int render_title(dvtext_t *dvtext, standard_t *standard, standardtype_t type) {
	int ret;
	char *output_filename = DEFAULT_OUTPUT_FILENAME;
	dv_encoder_t *encoder = NULL;
	avi_t *avifile = NULL;	
	uint8_t *inputbuffer;
	uint8_t *outputbuffer;
	int i;
	unsigned char *pixels[3];
	time_t current_time;

	/* initialize avilib */

	avifile = AVI_open_output_file(output_filename);
	if (avifile == NULL) {
		fprintf(stderr,"error: cannot open AVI file \"%s\"\n", output_filename);
		return -1;
	}

	
	AVI_set_video(avifile, 
		(standard+type)->width, 
		(standard+type)->height, 
		(standard+type)->framerate, 
		"DVSD");
	
	AVI_set_audio(avifile, 2, 32000, 16, 1, 0);

	/* initialize libdv encoder */

	encoder = dv_encoder_new(FALSE, FALSE, FALSE);
	if(type == PAL) 
		encoder->isPAL = TRUE;
	else
		encoder->isPAL = FALSE;
		
	encoder->is16x9 = FALSE;
	encoder->vlc_encode_passes = 3;
	encoder->static_qno = 0;
	encoder->force_dct = DV_DCT_AUTO;

	/* render! */

	int leadin_point = 0;
	int fadein_point = dvtext->title_leadin;
	int solid_point  = dvtext->title_leadin + dvtext->title_fadein;
	int fadeout_point = dvtext->title_leadin + dvtext->title_fadein + dvtext->title_solid;
	int leadout_point = dvtext->title_leadin + dvtext->title_fadein + dvtext->title_solid + dvtext->title_fadeout;
	int total_frames = dvtext->title_leadin + dvtext->title_fadein + dvtext->title_solid + dvtext->title_fadeout + dvtext->title_leadout;

	if((inputbuffer = (uint8_t *)malloc(sizeof(uint8_t)*((standard+type)->width)*((standard+type)->height)*4)) == NULL)
		return -1;
	
	if((outputbuffer = (uint8_t *)malloc(sizeof(uint8_t)*((standard+type)->width)*((standard+type)->height)*2)) == NULL)
		return -1;
	
	for (i=0; i<total_frames; i++) {
		memset(inputbuffer, 0, sizeof(uint8_t)*((standard+type)->width)*((standard+type)->height)*4);
		memset(outputbuffer, 0, sizeof(uint8_t)*((standard+type)->width)*((standard+type)->height)*2);

		/* iterate over text items, and draw them to the buffer */
		textitem_t *item;
		for (item=dvtext->textitem_list; item!=NULL; item=item->next) {
			/* calculate the xpos, for centering */
			int xpos = (standard+type)->width/2 - item->pixel_width/2;
			int alpha = 0xFF;

			if ((i>=leadin_point) && (i<fadein_point)) {
				alpha = 0;
			} else if ((i>=fadein_point) && (i<solid_point)) {
				alpha = ((i-fadein_point)/(float)dvtext->title_fadein) * 255;
			} else if ((i>=solid_point) && (i<fadeout_point)) {
				alpha = 255;
			} else if ((i>=fadeout_point) && (i<leadout_point)) {
				alpha = (1-((i-fadeout_point)/(float)dvtext->title_fadeout)) * 255;
			} else if (i>=leadout_point) {
				alpha = 0;
			}


			/* draw! */
			fonts_string_draw(
				inputbuffer,
				(standard+type)->width,
				(standard+type)->height,
				xpos,
				item->ypos,
				item->textface->face,
				item->string,
				alpha
			);

		}

		pixels[0] = inputbuffer;
		pixels[1] = NULL;
		pixels[2] = NULL;
		current_time = 0; //time(NULL);
		dv_encode_full_frame(encoder, pixels, e_dv_color_rgb, outputbuffer);
		dv_encode_metadata(outputbuffer, FALSE, FALSE, &current_time, 0);
		dv_encode_timecode(outputbuffer, FALSE, 0);
		if(type == NTSC)
		ret = AVI_write_frame(avifile, outputbuffer, DV_FRAME_SIZE_NTSC, /* keyframe? always! */ 1);
		if(type == PAL)
		ret = AVI_write_frame(avifile, outputbuffer, DV_FRAME_SIZE_PAL, /* keyframe? always! */ 1);
		if(type == SECAM)
		ret = AVI_write_frame(avifile, outputbuffer, DV_FRAME_SIZE_PAL, /* keyframe? always! */ 1);
		
		if (ret < 0) {
			AVI_print_error("avi video write error");
			return -1;
		}

		int audiodata_size = 2 /*chan*/ * 2 /*bytes per samp*/ * 32000 /*samps/sec*/ / (float)((standard+type)->framerate) /*frames/sec*/ ;
		char* audiodata = (char*)malloc(audiodata_size);
		memset(audiodata, 0, audiodata_size);
		AVI_write_audio(avifile, audiodata, audiodata_size);
		if (ret < 0) {
			AVI_print_error("avi audio write error");
			return -1;
		}
		if(audiodata) free(audiodata);
		
	}

	/* clean up avilib */
	AVI_close(avifile);

	/* free buffers */
	if(inputbuffer) free(inputbuffer);
	if(outputbuffer) free(outputbuffer);

	/* clean up libdv encoder */
	dv_encoder_free(encoder);
}


/*
 * render_scroll()
 *
 * Input:
 *	dvtext = the dvtext_t handle for this job
 *
 * Render a scrolling series of text strings, suitable
 * for closing credits.
 */

int render_scroll(dvtext_t *dvtext, standard_t *standard, standardtype_t type) {
	int ret;
	char *output_filename = DEFAULT_OUTPUT_FILENAME;
	dv_encoder_t *encoder = NULL;
	avi_t *avifile = NULL;
	uint8_t *inputbuffer;
	uint8_t *outputbuffer;
	int i;
	unsigned char *pixels[3];
	time_t current_time;

	/* initialize avilib */

	avifile = AVI_open_output_file(output_filename);
	if (avifile == NULL) {
		fprintf(stderr,"error: cannot open AVI file \"%s\"\n", output_filename);
		return -1;
	}

	AVI_set_video(avifile, 
		(standard+type)->width, 
		(standard+type)->height, 
		(standard+type)->framerate, 
		"DVSD");
		
	AVI_set_audio(avifile, 2, 32000, 16, 1, 0);

	/* initialize libdv encoder */

	encoder = dv_encoder_new(FALSE, FALSE, FALSE);
	if(type == PAL) 
		encoder->isPAL = TRUE;
	else
		encoder->isPAL = FALSE;
		
	encoder->is16x9 = FALSE;
	encoder->vlc_encode_passes = 3;
	encoder->static_qno = 0;
	encoder->force_dct = DV_DCT_AUTO;

	/* render! */

	/* determine the total y-pixel distance to cover */
	int total_y = 0;
	textitem_t *item;
	for (item=dvtext->textitem_list; item!=NULL; item=item->next) {
		total_y += item->pointsize + DEFAULT_SCROLL_YSPACING;
		/* calculate the ypos offset of each item */
		item->ypos = total_y;
	}

	int speedstep = 2;

	int total_frames = (total_y + (standard+type)->height) / speedstep;
	
	if((inputbuffer = (uint8_t *)malloc(sizeof(uint8_t)*((standard+type)->width)*((standard+type)->height)*4)) == NULL)
		return -1;
	
	if((outputbuffer = (uint8_t *)malloc(sizeof(uint8_t)*((standard+type)->width)*((standard+type)->height)*2)) == NULL)
		return -1;
	
	for (i=0; i<total_frames; i++) {
		memset(inputbuffer, 0, sizeof(uint8_t)*((standard+type)->width)*((standard+type)->height)*4);
		memset(outputbuffer, 0, sizeof(uint8_t)*((standard+type)->width)*((standard+type)->height)*2);

		/* iterate over text items, and draw them to the buffer */
		for (item=dvtext->textitem_list; item!=NULL; item=item->next) {
			/* calculate the xpos, for centering */
			int xpos = ((standard+type)->width)/2 - item->pixel_width/2;
			int alpha = 0xFF;

			int ypos = (standard+type)->height + item->ypos - i*speedstep;

			if ((ypos < 0) || (ypos > ((standard+type)->height) + item->pointsize + 20 )) {
				continue;
			}

			/* draw! */
			fonts_string_draw(
				inputbuffer,
				(standard+type)->width,
				(standard+type)->height,
				xpos,
				ypos,
				item->textface->face,
				item->string,
				alpha
			);

		}

		pixels[0] = inputbuffer;
		pixels[1] = NULL;
		pixels[2] = NULL;
		current_time = time(NULL);
		dv_encode_full_frame(encoder, pixels, e_dv_color_rgb, outputbuffer);
		dv_encode_metadata(outputbuffer, FALSE, FALSE, &current_time, 0);
		dv_encode_timecode(outputbuffer, FALSE, 0);
		if(type == NTSC)
		ret = AVI_write_frame(avifile, outputbuffer, DV_FRAME_SIZE_NTSC, /* keyframe? always! */ 1);
		if(type == PAL)
		ret = AVI_write_frame(avifile, outputbuffer, DV_FRAME_SIZE_PAL, /* keyframe? always! */ 1);
		if(type == SECAM)
		ret = AVI_write_frame(avifile, outputbuffer, DV_FRAME_SIZE_PAL, /* keyframe? always! */ 1);

		if (ret < 0) {
			AVI_print_error("avi video write error");
			return -1;
		}

		int audiodata_size = 2 /*chan*/ * 2 /*bytes per samp*/ * 32000 /*samps/sec*/ / (float)((standard+type)->framerate) /*frames/sec*/ ;
		char* audiodata = (char*)malloc(audiodata_size);
		memset(audiodata, 0, audiodata_size);
		AVI_write_audio(avifile, audiodata, audiodata_size);
		if (ret < 0) {
			AVI_print_error("avi audio write error");
			return -1;
		}
		if(audiodata) free(audiodata);
	}

	/* clean up avilib */
	AVI_close(avifile);

	/* free buffers */
	if(inputbuffer) free(inputbuffer);
	if(outputbuffer) free(outputbuffer);

	/* clean up libdv encoder */
	dv_encoder_free(encoder);
}

